@@ -1,15 +1,14 @@
 import os
-import sys
 import time
 import json
-import requests
 from struct import pack, unpack
 from base64 import b64encode, b64decode
 
 import gw2api
 
 
-__all__ = ("encode_item_link", "encode_coin_link", "encode_chat_link", "decode_chat_link")
+__all__ = ("encode_item_link", "encode_coin_link",
+           "encode_chat_link", "decode_chat_link")
 
 
 def mtime(path):
@@ -27,7 +26,7 @@ def get_cached(path, cache_name=None, **kwargs):
     """
     if gw2api.cache_dir and gw2api.cache_time and cache_name is not False:
         if cache_name is None:
-            cache_name = path
+            cache_name = path.split("/")[-1]
         cache_file = os.path.join(gw2api.cache_dir, cache_name)
         if mtime(cache_file) >= time.time() - gw2api.cache_time:
             with open(cache_file, "r") as fp:
@@ -35,7 +34,21 @@ def get_cached(path, cache_name=None, **kwargs):
     else:
         cache_file = None
 
-    r = gw2api.session.get(gw2api.BASE_URL + path, **kwargs)
+    if path.startswith("http://") or path.startswith("https://"):
+        url = path
+    else:
+        url = gw2api.BASE_URL + path
+
+    r = gw2api.session.get(url, **kwargs)
+
+    if r.status_code == 500:
+        try:
+            response = r.json()
+        except ValueError:  # pragma: no cover
+            response = None
+        if isinstance(response, dict) and "text" in response:
+            r.reason = response["text"]
+
     r.raise_for_status()
     data = r.json()
 
@@ -46,7 +59,8 @@ def get_cached(path, cache_name=None, **kwargs):
     return data
 
 
-def encode_item_link(item_id, number=1, skin_id=None, upgrade1=None, upgrade2=None):
+def encode_item_link(item_id, number=1, skin_id=None,
+                     upgrade1=None, upgrade2=None):
     """Encode a chat link for an item (or a stack of items).
 
     :param item_id: the Id of the item
@@ -63,12 +77,14 @@ def encode_item_link(item_id, number=1, skin_id=None, upgrade1=None, upgrade2=No
 def encode_coin_link(copper, silver=0, gold=0):
     """Encode a chat link for an amount of coins.
     """
-    amount = copper + silver * 100 + gold * 100 * 100
-    return encode_chat_link(gw2api.TYPE_COIN, amount=amount)
+    return encode_chat_link(gw2api.TYPE_COIN, copper=copper, silver=silver,
+                            gold=gold)
 
 
 def encode_chat_link(link_type, **kwargs):
     if isinstance(link_type, basestring):
+        if link_type not in gw2api.LINK_TYPES:
+            raise Exception("Unknown link type '%s'" % link_type)
         link_type = gw2api.LINK_TYPES[link_type]
 
     if link_type == gw2api.TYPE_COIN:
@@ -111,11 +127,6 @@ def decode_chat_link(string):
 
     link_type, = unpack("<B", data[0])
 
-    link_type_string = None
-    for key, value in gw2api.LINK_TYPES.iteritems():
-        if value == link_type:
-            link_type_string = key
-
     if link_type == gw2api.TYPE_COIN:
         amount, = unpack("<I", data[1:])
         return "coin", {"amount": amount}
@@ -124,7 +135,7 @@ def decode_chat_link(string):
         number, item_id = unpack("<BI", data[1:6])
         flags = (item_id & 0xFF000000) >> 24
         item_id &= 0x00FFFFFF
-        values = {"number": number, "id": item_id, "flags": flags}
+        values = {"number": number, "id": item_id}
         o = 6
         if flags & 0x80:
             values["skin_id"], = unpack("<I", data[o:o+4])
@@ -137,8 +148,13 @@ def decode_chat_link(string):
             o += 4
         return "item", values
 
+    link_type_string = None
+    for key, value in gw2api.LINK_TYPES.iteritems():
+        if value == link_type:
+            link_type_string = key
+
     if link_type in (gw2api.TYPE_TEXT, gw2api.TYPE_MAP, gw2api.TYPE_SKILL,
                      gw2api.TYPE_TRAIT, gw2api.TYPE_RECIPE,
                      gw2api.TYPE_SKIN, gw2api.TYPE_OUTFIT):
         id, = unpack("<I", data[1:])
-        return link_type_string, { "id": id }
+        return link_type_string, {"id": id}
